﻿<#
.SYNOPSIS
    Prépare les paquets d'application Intune (.intunewin) pour Antidote 11.
.DESCRIPTION
    Ce script prépare les paquets d'application Intune pour Antidote 11.
    Il prépare également les scripts de détection à utiliser avec les paquets d'application.
    Ce script créera trois paquets d'application (.intunewin) :
        - Antidote11.intunewin pour Antidote 11.
        - PrecedentsAntidote.intunewin pour la désinstallation des éditions précédentes.
        - At.intunewin pour la désinstallation de l'Assistant de téléchargement.
.NOTES
    Ce script utilise le « Microsoft Win32 Content Prep Tool » pour générer les paquets au format .intunewin.
    Cet outil nécessite le .NET Framework 4.7.2 ou plus récent.
    Pour plus d'information sur cet outil, voir la page du projet : https://github.com/microsoft/Microsoft-Win32-Content-Prep-Tool.
.LINK
    https://www.antidote.info/antidote-11/documentation/installation-multiposte-windows/deploiement-par-microsoft-intune
.EXAMPLE
    .\Creer-PaquetIntune.ps1
    Création des paquets à partir d'une installation extraite.
#>

[CmdletBinding(SupportsShouldProcess)]
param ()

trap [System.Management.Automation.ValidationMetadataException] {
    Write-Host "Une erreur est survenue durant la lecture des fichiers d'installation."
    Write-Host 'Appuyez sur une touche pour continuer...'
    $host.UI.RawUI.ReadKey() | Out-Null
    break
}

enum InstallationType {
    ANTIDOTE_ML = 0
    ANTIDOTE_FR = 1
    ANTIDOTE_EN = 2
    CONNECTIX = 3
}

$InstallerPackages = Get-ChildItem -Path "$PSScriptRoot\msi\druide\*.msi" -ErrorAction Ignore
$HasConnectix = $false
$HasEnMod = $false
$HasModFr = $false
$HasAntidote = $false
switch ($InstallerPackages.Name) {
    'Antidote-Connectix11.msi' { $HasConnectix = $true }
    'Antidote11-English-module.msi' { $HasEnMod = $true }
    'Antidote11-Module-francais.msi' { $HasModFr = $true }
    'Antidote11.msi' { $HasAntidote = $true }
}

Remove-Variable Type -WhatIf:$false -ErrorAction Ignore
$Type = $null
if ($HasConnectix) {
    if ($HasAntidote) {
        if ($HasModFr -and $HasEnMod) {
            $Type = [InstallationType]::ANTIDOTE_ML
        } elseif ($HasModFr) {
            $Type = [InstallationType]::ANTIDOTE_FR
        } elseif ($HasEnMod) {
            $Type = [InstallationType]::ANTIDOTE_EN
        }
    } elseif (-not ($HasModFr -or $HasEnMod)) {
        $Type = [InstallationType]::CONNECTIX
    }
}

if ($null -eq $Type) {
    if (-not $HasConnectix -and -not $HasEnMod -and -not $HasModFr -and -not $HasAntidote) {
        Write-Host "Aucun fichier d'installation (.msi) d'Antidote 11 n'a été trouvé."
    } else {
        Write-Host "Un ou des fichiers d'installation (.msi) sont manquants pour une installation fonctionnelle d'Antidote 11."
    }
    Write-Host 'Assurez-vous d''extraire les scripts dans le dossier extrait par l''installateur, à côté de "Inst - Antidote.exe".'
    Write-Host 'Ce dossier devrait contenir les sous-dossiers "Documentation" et "msi".'
    Write-Host 'Appuyez sur une touche pour continuer...'
    $host.UI.RawUI.ReadKey() | Out-Null
    exit 1
}

if ($Type -eq [InstallationType]::CONNECTIX) {
    $CommonInstaller = 'Antidote-Connectix11.msi'
} else {
    $CommonInstaller = 'Antidote11.msi'
}

Write-Host "Le présent script est réservé à l'usage exclusif des organisations signataires d'une convention de licence régissant l'utilisation multiposte du logiciel Antidote."

if ( -not $(Test-Path -Path "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool\IntuneWinAppUtil.exe" -PathType Leaf )) {
    if ( $(Test-Path -Path "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool") ) {
        Remove-Item -Path "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool" -Recurse -Force
    }

    $LicenceAcceptedMarker = "$env:APPDATA\Druide\IntunePrepAccept"
    $Accepted = (Test-Path -Path $LicenceAcceptedMarker)

    while ( -not $Accepted) {
        $Titre = "Ce script utilise le Microsoft Win32 Content Prep Tool : https://github.com/microsoft/Microsoft-Win32-Content-Prep-Tool `n`nAvant d'utiliser cet outil, vous devez lire et accepter les termes de licences de l'outil ainsi que la politique de confidentialité et des données personnelles dans Intune."
        $Prompt = 'Acceptez-vous?'
        $Choices = @(
            [System.Management.Automation.Host.ChoiceDescription]::new('&Oui', 'Vous avez lu et acceptez.'),
            [System.Management.Automation.Host.ChoiceDescription]::new('&Non', 'Vous refusez. Vous ne pourrez pas déployer Antidote dans Intune avec ce script.'),
            [System.Management.Automation.Host.ChoiceDescription]::new('&Toujours', "Vous avez lu et acceptez. Votre réponse sera enregistrée avec le fichier $LicenceAcceptedMarker."),
            [System.Management.Automation.Host.ChoiceDescription]::new('&Afficher les termes', 'Afficher les termes de la licence du Microsoft Win32 Content Prep Tool dans votre navigateur.'),
            [System.Management.Automation.Host.ChoiceDescription]::new('Afficher la politique de &confidentialité', 'Afficher la politique de confidentialité et des données personnelles dans Intune dans votre navigateur.')
        )
        $Default = 1

        $Choice = $host.UI.PromptForChoice($Titre, $Prompt, $Choices, $Default)
        switch ($Choice) {
            2 {
                if ( -not (Test-Path -Path (Split-Path -Path $LicenceAcceptedMarker -Parent))) {
                    New-Item -Path (Split-Path -Path $LicenceAcceptedMarker -Parent) -ItemType Directory | Out-Null
                }
                New-Item -Path $LicenceAcceptedMarker -ItemType File | Out-Null
            }
            { $_ -eq 0 -or $_ -eq 2 } {
                $Accepted = $true
                break
            }
            1 {
                exit 1
            }
            3 {
                Start-Process 'https://github.com/microsoft/Microsoft-Win32-Content-Prep-Tool/blob/master/Microsoft%20License%20Terms%20For%20Win32%20Content%20Prep%20Tool.pdf'
            }
            4 {
                Start-Process 'https://learn.microsoft.com/mem/intune/protect/privacy-personal-data'
            }
        }
    }

    $LatestRelease = Invoke-WebRequest https://api.github.com/repos/microsoft/Microsoft-Win32-Content-Prep-Tool/releases/latest | ConvertFrom-Json
    Invoke-WebRequest $LatestRelease.zipball_url -OutFile "$PSScriptRoot\MsIntunePrep.zip"
    Expand-Archive "$PSScriptRoot\MsIntunePrep.zip" -DestinationPath "$PSScriptRoot"
    Get-Item -Path "$PSScriptRoot\microsoft-Microsoft-Win32-Content-Prep-Tool-*" | Where-Object { $_.Attributes -eq 'Directory' } | Select-Object -First 1 | Rename-Item -NewName 'Microsoft-Win32-Content-Prep-Tool'

    Remove-Item -Path "$PSScriptRoot\MsIntunePrep.zip"
}

Remove-Variable AntidoteVersion -WhatIf:$false -ErrorAction:Ignore
Remove-Variable ConnectixVersion -WhatIf:$false -ErrorAction:Ignore
$Installer = New-Object -ComObject WindowsInstaller.Installer
$Database = $Installer.OpenDatabase((Join-Path -Path "$PSScriptRoot\msi\druide" -ChildPath $CommonInstaller), 0)
if ($null -ne $Database) {
    $View = $Database.OpenView("SELECT Value FROM Property WHERE Property = 'ProductVersion'")
    $View.Execute()
    $Record = $View.Fetch()
    $ProductVersion = $Record.StringData(1)
    $View.Close()
    [System.Runtime.InteropServices.Marshal]::ReleaseComObject($Record) | Out-Null
    [System.Runtime.InteropServices.Marshal]::ReleaseComObject($View) | Out-Null
    [System.Runtime.InteropServices.Marshal]::ReleaseComObject($Database) | Out-Null
    Remove-Variable View, Record -WhatIf:$false
}
if ($Type -eq [InstallationType]::CONNECTIX) {
    [ValidateNotNullOrEmpty()] [string]$ConnectixVersion = $ProductVersion
} else {
    [ValidateNotNullOrEmpty()] [string]$AntidoteVersion = $ProductVersion
    [ValidateNotNullOrEmpty()] [string]$ConnectixVersion = $AntidoteVersion
}
$UpdatePackages = Get-ChildItem -Path "$PSScriptRoot\msi\druide\Diff_Connectix_11*.msp" -File
foreach ($UpdatePackage in $UpdatePackages) {
    $Node = $Installer.ExtractPatchXMLData($UpdatePackage.FullName) | Select-Xml -XPath "//msi:TargetProduct[msi:TargetVersion='$ConnectixVersion']/msi:UpdatedVersion" -Namespace @{msi = 'http://www.microsoft.com/msi/patch_applicability.xsd' }
    $Version = $Node.Node.InnerText
    if ($null -ne $Version -and [System.Version]$Version -gt [System.Version]$ConnectixVersion) {
        $ConnectixVersion = $Version
    }
}
[System.Runtime.InteropServices.Marshal]::ReleaseComObject($Installer) | Out-Null
Remove-Variable Installer, Database -WhatIf:$false

$Encoding = 'UTF8'
# UTF8 dans Windows PowerShell 5.1 equivaut à UTF8BOM dans PowerShell 6 et plus.
if ( $PSVersionTable.PSVersion.Major -ge 6) {
    $Encoding = 'utf8BOM'
}

(Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1") -replace '(\[InstallationType\]\$Type) = .+', "`$1 = '$Type'" | Set-Content -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1" -Encoding $Encoding
(Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1") -replace '(\[InstallationType\]\$Type) = .+', "`$1 = '$Type'" | Set-Content -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1" -Encoding $Encoding

if ($Type -ne [InstallationType]::CONNECTIX) {
    (Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1") -replace '(\[string\]\$AntidoteVersion) = .+', "`$1 = '$AntidoteVersion'" | Set-Content -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1" -Encoding $Encoding
    (Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1") -replace '(\[string\]\$AntidoteVersion) = .+', "`$1 = '$AntidoteVersion'" | Set-Content -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1" -Encoding $Encoding
}

(Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1") -replace '(\[string\]\$ConnectixVersion) = .+', "`$1 = '$ConnectixVersion'" | Set-Content -Path "$PSScriptRoot\msi\druide\Installer-Antidote.ps1" -Encoding $Encoding
(Get-Content -Encoding $Encoding -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1") -replace '(\[string\]\$ConnectixVersion) = .+', "`$1 = '$ConnectixVersion'" | Set-Content -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1" -Encoding $Encoding

if ($(Test-Path -Path "$PSScriptRoot\Antidote11.intunewin")) {
    Remove-Item -Path "$PSScriptRoot\Antidote11.intunewin"
}
$Antidote11Arguments = @(
    '-c', """$PSScriptRoot\msi\druide""", '-s', $CommonInstaller, '-o', """$PSScriptRoot"""
)
if ($PSCmdlet.ShouldProcess("$Antidote11Arguments", 'IntuneWinAppUtil.exe')) {
    $Process = Start-Process -FilePath "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool\IntuneWinAppUtil.exe" -ArgumentList $Antidote11Arguments -Wait -NoNewWindow -PassThru
    if ($Process.ExitCode -ne 0) {
        Write-Error "Erreur $($Process.ExitCode)"
        Write-Host 'Appuyez sur une touche pour continuer...'
        $host.UI.RawUI.ReadKey() | Out-Null
        exit 1
    }
}
Copy-Item -Path "$PSScriptRoot\msi\Detecter-Antidote.ps1" -Destination "$PSScriptRoot" -Force
if ($CommonInstaller -ne 'Antidote11.msi') {
    Rename-Item -Path "$PSScriptRoot\$($CommonInstaller -replace '(.+)\.msi', '$1').intunewin" -NewName 'Antidote11.intunewin'
}

if ($(Test-Path -Path "$PSScriptRoot\At.intunewin")) {
    Remove-Item -Path "$PSScriptRoot\At.intunewin"
}
$AtArguments = @(
    '-c', """$PSScriptRoot\msi\AT""", '-s', 'Installer-At.ps1', '-o', """$PSScriptRoot"""
)
if ($PSCmdlet.ShouldProcess("$AtArguments", 'IntuneWinAppUtil.exe')) {
    $Process = Start-Process -FilePath "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool\IntuneWinAppUtil.exe" -ArgumentList $AtArguments -Wait -NoNewWindow -PassThru
    if ($Process.ExitCode -ne 0) {
        Write-Error "Erreur $($Process.ExitCode)"
        Write-Host 'Appuyez sur une touche pour continuer...'
        $host.UI.RawUI.ReadKey() | Out-Null
        exit 1
    }
}
Copy-Item -Path "$PSScriptRoot\msi\Detecter-At.ps1" -Destination "$PSScriptRoot" -Force
Rename-Item -Path "$PSScriptRoot\Installer-At.intunewin" -NewName 'At.intunewin'

if ($(Test-Path -Path "$PSScriptRoot\PrecedentsAntidote.intunewin")) {
    Remove-Item -Path "$PSScriptRoot\PrecedentsAntidote.intunewin"
}
$PrecedentsAntidoteArguments = @(
    '-c', """$PSScriptRoot\msi\PrecedentsAntidote""", '-s', 'Installer-PrecedentsAntidote.ps1', '-o', """$PSScriptRoot"""
)
if ($PSCmdlet.ShouldProcess("$PrecedentsAntidoteArguments", 'IntuneWinAppUtil.exe')) {
    $Process = Start-Process -FilePath "$PSScriptRoot\Microsoft-Win32-Content-Prep-Tool\IntuneWinAppUtil.exe" -ArgumentList $PrecedentsAntidoteArguments -Wait -NoNewWindow -PassThru
    if ($Process.ExitCode -ne 0) {
        Write-Error "Erreur $($Process.ExitCode)"
        Write-Host 'Appuyez sur une touche pour continuer...'
        $host.UI.RawUI.ReadKey() | Out-Null
        exit 1
    }
}
Copy-Item -Path "$PSScriptRoot\msi\Detecter-PrecedentsAntidote.ps1" -Destination "$PSScriptRoot" -Force
Rename-Item -Path "$PSScriptRoot\Installer-PrecedentsAntidote.intunewin" -NewName 'PrecedentsAntidote.intunewin'

Write-Host "La création des paquets .intunewin s’est terminée correctement."
