﻿[CmdletBinding(SupportsShouldProcess)]
param ()

trap [System.Management.Automation.ValidationMetadataException] {
    Write-Error "Ce script doit être adapté avant d'être utilisé. Il doit inclure des numéros de version valide pour Antidote 11 et Connectix 11."
    Write-Error "Ces ajustements peuvent être faits en lançant Creer-PaquetIntune.ps1."
    break
}

enum InstallationType {
    ANTIDOTE_ML = 0
    ANTIDOTE_FR = 1
    ANTIDOTE_EN = 2
    CONNECTIX = 3
}

[ValidateNotNullOrEmpty()] [InstallationType]$Type = $null
if ($Type -ne [InstallationType]::CONNECTIX) {
    [ValidatePattern('^11\.\d\.\d{1,4}$')] [string]$AntidoteVersion = $null
} else {
    Remove-Variable AntidoteVersion -WhatIf:$false -ErrorAction Ignore
}
[ValidatePattern('^11\.\d\.\d{1,4}$')] [string]$ConnectixVersion = $null

$SourcePath = $PSScriptRoot
$DestinationPath = 'C:\Program Files\Druide'
$LogPath = $(Join-Path $env:ProgramData 'Druide')
$ScriptLog = $(Join-Path $LogPath 'logAntidote11.txt')

$Products = @(
    [PSCustomObject]@{
        Name          = 'Antidote 11'
        Code          = '{2643823D-D15F-4046-8388-401756A5C921}'
        Version       = [System.Version]$AntidoteVersion
        IsRequired    = $Type -ne [InstallationType]::CONNECTIX
        InstallerPath = "$SourcePath\Antidote11.msi"
        Transforms    = "$SourcePath\ReseauAntidote.mst;$SourcePath\Antidote11-Interface-fr.mst"
        LogFile       = 'Antidote11.txt'
        AlwaysRestart = $false
    },
    [PSCustomObject]@{
        Name          = 'Antidote 11 - Module Français'
        Code          = '{2643823D-D15F-4046-8388-401756A5C922}'
        Version       = [System.Version]$AntidoteVersion
        IsRequired    = $Type -in [InstallationType]::ANTIDOTE_FR, [InstallationType]::ANTIDOTE_ML
        InstallerPath = "$SourcePath\Antidote11-Module-francais.msi"
        Transforms    = "$SourcePath\Antidote11-Module-francais-Interface-fr.mst"
        LogFile       = 'Antidote11_Module_Français.txt'
        AlwaysRestart = $false
    },
    [PSCustomObject]@{
        Name          = 'Antidote 11 - English Module'
        Code          = '{2643823D-D15F-4046-8388-401756A5C923}'
        Version       = [System.Version]$AntidoteVersion
        IsRequired    = $Type -in [InstallationType]::ANTIDOTE_EN, [InstallationType]::ANTIDOTE_ML
        InstallerPath = "$SourcePath\Antidote11-English-module.msi"
        Transforms    = "$SourcePath\Antidote11-English-module-Interface-fr.mst"
        LogFile       = 'Antidote11_English_Module.txt'
        AlwaysRestart = $false
    },
    [PSCustomObject]@{
        Name          = 'Antidote - Connectix 11'
        Code          = '{2643823D-D15F-4046-8388-401756A5C924}'
        Version       = [System.Version]$ConnectixVersion
        IsRequired    = $true
        InstallerPath = "$SourcePath\Antidote-Connectix11.msi"
        Transforms    = "$SourcePath\ReseauConnectix.mst;$SourcePath\Antidote-Connectix11-Interface-fr.mst"
        LogFile       = 'Antidote11_Connectix.txt'
        AlwaysRestart = $true
        Updates       = "$SourcePath\Diff_Connectix_11*.msp"
    }
)
foreach ($Product in $Products) {
    Add-Member -MemberType ScriptProperty -Name 'IsPresent' -Value { Test-Path -Path $this.InstallerPath } -InputObject $Product
}

enum MsiExecErrorCode {
    ERROR_SUCCESS = 0
    ERROR_SUCCESS_REBOOT_INITIATED = 1641
    ERROR_SUCCESS_REBOOT_REQUIRED = 3010
    ERROR_INSTALL_ALREADY_RUNNING = 1618
}

$SuccessCodes = @(
    [MsiExecErrorCode]::ERROR_SUCCESS,
    [MsiExecErrorCode]::ERROR_SUCCESS_REBOOT_REQUIRED
)

$MinorUpgradeArguments = @(
    'REINSTALLMODE=vomus',
    'REINSTALL=ALL',
    'DRUIDE_MAJ=1'
)

$ShouldRestart = $false
$ForcedRestart = $false

New-Item -ItemType Directory -Force -Path $LogPath

$InstallationProduct = "Antidote 11"
if ($Type -eq [InstallationType]::CONNECTIX) {
    $InstallationProduct = "Connectix 11"
}

Add-Content -Path $ScriptLog -Value "Déploiement d'$InstallationProduct"

if ($Products.Where({ $_.IsRequired -and -not $_.IsPresent })) {
    Add-Content -Path $ScriptLog -Value "Certains des fichiers d'installation nécessaires sont manquants. L'installation d'$InstallationProduct a été annulée."
    foreach ($Product in $Products.Where({ $_.IsRequired -and -not $_.IsPresent })) {
        Add-Content -Path $ScriptLog -Value "$($Product.InstallerPath) est manquant."
    }
    exit 1
}

$Installer = New-Object -ComObject WindowsInstaller.Installer

foreach ($Product in $Products) {
    $UpdatePackages = $null
    if ($null -ne $Product.Updates) {
        $UpdatePackages = Get-ChildItem -Path $Product.Updates -File
    }

    # On cherche dans tout les contextes d'installation possible :
    # MSIINSTALLCONTEXT_USERMANAGED + MSIINSTALLCONTEXT_USERUNMANAGED + MSIINSTALLCONTEXT_MACHINE = 7
    $InstalledProducts = $Installer.ProductsEx($Product.Code, '', 7)
    $IsInstalled = $InstalledProducts.Count() -gt 0
    $InstalledVersion = $null
    if ($IsInstalled) {
        [System.Version]$InstalledVersion = $InstalledProducts[0].InstallProperty('VersionString')
    }

    # On désinstalle les produits non nécessaires et ceux qui sont trop récents (donc pas installés par Intune)
    if ($IsInstalled -and
        (-not $Product.IsRequired -or $InstalledVersion -gt $Product.Version)) {
        $UninstallArguments = @(
            '/qn',
            '/x',
            $Product.Code,
            'MSIRESTARTMANAGERCONTROL=DisableShutdown',
            'REBOOT=ReallySuppress',
            '/l*v',
            """$LogPath\logDes$($Product.LogFile)"""
        )
        Add-Content -Path $ScriptLog -Value "Désinstallation de $($Product.Name) ($InstalledVersion) : $UninstallArguments"
        if ($PSCmdlet.ShouldProcess("$($Product.Name) ($InstalledVersion) : $UninstallArguments", 'Désinstallation')) {
            $Process = Start-Process 'msiexec.exe' -ArgumentList $UninstallArguments -Wait -NoNewWindow -PassThru
            if ($Process.ExitCode -notin $SuccessCodes) {
                exit $Process.ExitCode
            } elseif ($Process.ExitCode -eq [MsiExecErrorCode]::ERROR_SUCCESS_REBOOT_REQUIRED -or $Product.AlwaysRestart) {
                $ShouldRestart = $true
            }
        }
        $IsInstalled = $false
        $InstalledVersion = $null
    }

    # On installe la version du produit incluse dans ce paquet
    if ($Product.IsRequired) {
        if (-not $IsInstalled -or $InstalledVersion -ne $Product.Version) {
            $InstallArguments = @(
                '/qn',
                '/i',
                """$($Product.InstallerPath)""",
                "TRANSFORMS=""$($Product.Transforms)""",
                'REBOOT=ReallySuppress',
                'MSIRESTARTMANAGERCONTROL=DisableShutdown',
                '/l*v',
                """$LogPath\logIns$($Product.LogFile)"""
            )

            # Ajout à la commande, si une mise à jour est appliquée (produit déjà installé ou non)
            if ($null -ne $UpdatePackages) {
                $InstallArguments += '/update'
                $UpdateArgument = $null
                foreach ($Update in $UpdatePackages) {
                    if ($null -ne $UpdateArgument) {
                        $UpdateArgument += ';'
                    }
                    $UpdateArgument += $Update.FullName
                }
                $InstallArguments += """$UpdateArgument"""
            }

            $Operation = 'Installation'
            # Ajout à la commande, pour une mise à jour d'un produit déjà installé
            if ($IsInstalled) {
                $InstallArguments += $MinorUpgradeArguments
                if ($Product.AlwaysRestart) {
                    $ForcedRestart = $true
                }
                $Operation = 'Mise à jour'
            }

            Add-Content -Path $ScriptLog -Value "$Operation de $($Product.Name) ($($Product.Version)) : $InstallArguments"
            if ($PSCmdlet.ShouldProcess("$($Product.Name) ($($Product.Version)) : $InstallArguments", $Operation)) {
                $Process = Start-Process 'msiexec.exe' -ArgumentList $InstallArguments -Wait -NoNewWindow -PassThru
                if ($Process.ExitCode -notin $SuccessCodes) {
                    exit $Process.ExitCode
                } elseif ($Product.AlwaysRestart -or
                            ($Process.ExitCode -eq [MsiExecErrorCode]::ERROR_SUCCESS_REBOOT_REQUIRED)) {
                    $ShouldRestart = $true
                }
            } elseif ($Product.AlwaysRestart) {
                $ShouldRestart = $true
            }
        } else {
            Add-Content -Path $ScriptLog -Value "$($Product.Name) ($($Product.Version)) est déjà installé."
        }
    }
}

Add-Content -Path $ScriptLog -Value "Déploiement d'$InstallationProduct terminé"

if ($ShouldRestart) {
    if ($ForcedRestart) {
        exit [int][MsiExecErrorCode]::ERROR_SUCCESS_REBOOT_INITIATED
    } else {
        exit [int][MsiExecErrorCode]::ERROR_SUCCESS_REBOOT_REQUIRED
    }
} else {
    exit [int][MsiExecErrorCode]::ERROR_SUCCESS
}